'****************************************************************************
'*
'*                  Copyright (c) 1991-1994 by Jeffrey Sax
'*                            All rights reserved
'*                        Published and Distributed by
'*                           Willmann-Bell, Inc.
'*                             P.O. Box 35025
'*                        Richmond, Virginia 23235
'*                Voice (804) 320-7016 FAX (804) 272-5920
'*
'*
'*                NOTICE TO COMMERCIAL SOFTWARE DEVELOPERS
'*
'*        Prior to distributing software incorporating this code
'*        you MUST write Willmann-Bell, Inc. at the above address
'*        for validation of your book's (Astronomical Algorithms
'*        by Jean Meeus) and software Serial Numbers.  No additional
'*        fees will be required BUT you MUST have the following
'*        notice at the start of your program(s):
'*
'*                    This program contains code
'*              Copyright (c) 1991-1994 by Jeffrey Sax
'*              and Distributed by Willmann-Bell, Inc.
'*                         Serial #######
'*
'****************************************************************************

' First, some general constants and functions

CONST pi = 3.141592653589793#
CONST pi2 = (2 * pi)
CONST DToR = (pi / 180)
CONST HToR = (pi / 12)
CONST RToD = (180 / pi)
CONST RToH = (12 / pi)
CONST SToR = (DToR / 3600)
CONST EarthRadius = 6378.14
CONST SolarParallax = (8.794 * SToR)
CONST FK4System = 1
CONST FK5System = 2

'Rectangular coordinates

'Spherical coordinates
TYPE TSVECTOR
  l AS DOUBLE
  b AS DOUBLE
  r AS DOUBLE
END TYPE

'
'Basic support functions
'

DECLARE FUNCTION asin# (x#)
DECLARE FUNCTION acos# (x#)
DECLARE FUNCTION atn2# (y#, x#)
DECLARE FUNCTION log10# (x#)
DECLARE FUNCTION modpi2# (x#)
DECLARE FUNCTION modpi# (x#)
DECLARE SUB SphToRect (s AS TSVECTOR, P#())
DECLARE SUB RectToSph (P#(), s AS TSVECTOR)
DECLARE SUB CalcSinCosTab (x#, Degree%, SinTab#(), CosTab#())
DECLARE FUNCTION Interpol3# (y1#, y2#, y3#, n#)
DECLARE FUNCTION EvalPoly# (P#(), n%, x#)
DECLARE SUB XRot (v() AS DOUBLE, c AS DOUBLE, s AS DOUBLE, w() AS DOUBLE)
DECLARE SUB ZRot (v() AS DOUBLE, c AS DOUBLE, s AS DOUBLE, w() AS DOUBLE)

'
' Date / time functions
'

CONST secToT = (1 / (86400! * 36525!))
CONST TJ2000 = 0
CONST TB1950 = (2433282.4235# - 2451545) / 36525

TYPE TDATE
  yy AS INTEGER
  mm AS INTEGER
  dd AS DOUBLE
  H AS INTEGER
  m AS INTEGER
  s AS SINGLE
  END TYPE

DECLARE FUNCTION ApproxDeltaT (T#)
DECLARE FUNCTION CalendarToJD# (Date AS TDATE)
DECLARE FUNCTION DayOfWeek% (JD#)
DECLARE FUNCTION DayOfYear% (Date AS TDATE)
DECLARE SUB DateOfEaster (Year%, Date AS TDATE)
DECLARE FUNCTION EquationOfTime# (T#, RA#, Obl#, NutLon#)
DECLARE SUB JDToCalendar (JD#, Date AS TDATE)
DECLARE FUNCTION JDToT# (JD#)
DECLARE FUNCTION SiderealTime# (T#)
DECLARE FUNCTION SiderealTime0# (T#)
DECLARE FUNCTION TToJD# (T#)
DECLARE FUNCTION YearToJD0# (Year%)

'
' Position calculations
'

CONST GaussConstant = .01720209895#
CONST LightTimeConst = .0057755183# / 36525
CONST Obl2000 = 23.43929111# * DToR
CONST EPJ2000 = 0
CONST EPDATE = 3

TYPE TORBITEL
  T0 AS DOUBLE
  M0 AS DOUBLE
  a AS DOUBLE
  q AS DOUBLE
  n AS DOUBLE
  e AS DOUBLE
  LonPeri AS DOUBLE
  ArgPeri AS DOUBLE
  LonNode AS DOUBLE
  Incl AS DOUBLE
  MagParam1 AS SINGLE
  MagParam2 AS SINGLE
END TYPE

TYPE TORBITCON
  a AS DOUBLE
  b AS DOUBLE
  c AS DOUBLE
  aa AS DOUBLE
  bb AS DOUBLE
  cc AS DOUBLE
END TYPE

DECLARE SUB ObserverCoord (Phi#, Height#, RhoCosPhi#, RhoSinPhi#)
DECLARE SUB InitPlanetaryTerms (FilePath AS STRING)
DECLARE SUB FreePlanetaryTerms ()
DECLARE SUB ReadPlanetaryTerms (Planet%)
DECLARE SUB SunPosHi (T#, RefFrame%, s AS TSVECTOR)
DECLARE SUB SunRectCo (T#, RefFrame%, Obl#, r#())
DECLARE SUB PlPosHi (Planet%, T#, s AS TSVECTOR)
DECLARE SUB PlanetPosHi (Planet%, T#, s AS TSVECTOR)
DECLARE SUB PlutoPos (T#, s AS TSVECTOR)
DECLARE SUB HelioToGeo (SHelio AS TSVECTOR, SEarth AS TSVECTOR, SGeo AS TSVECTOR)
DECLARE SUB ReduceEclElements (T#, T1#, OrbitEl AS TORBITEL)
DECLARE SUB QuickReduceEclElements (OrbitEl AS TORBITEL)
DECLARE SUB FK4FK5ReduceEclElements (OrbitEl AS TORBITEL)
DECLARE FUNCTION Kepler# (m#, e#)
DECLARE FUNCTION Parabola# (dT#, q#)
DECLARE FUNCTION NearParabola# (dT#, q#, e#)
DECLARE SUB PlanetOrbitEl (T#, Planet%, Equinox%, OrbitEl AS TORBITEL)
DECLARE SUB PlanetPosLo (T#, Planet%, Equinox%, Obl#, r#())
DECLARE SUB CalcOrbitCon (OrbitEl AS TORBITEL, Obl#, OrbitCon AS TORBITCON)
DECLARE SUB PosRectCo (T#, OrbitEl AS TORBITEL, OrbitCon AS TORBITCON, r#())
DECLARE SUB RectHelioToGeo (RHelio#(), RSun#(), SGeo AS TSVECTOR)
DECLARE SUB CalcMoonAngles (T#, Om#, d#, m#, Md#, F#, e#)
DECLARE SUB MoonPos (T#, s AS TSVECTOR)

'Satellites of Jupiter

DECLARE SUB JSatViewFrom (n AS INTEGER, v() AS DOUBLE, vsOrigin AS TSVECTOR, vDummy() AS DOUBLE, bScale AS INTEGER, w() AS DOUBLE)
DECLARE SUB JSatEclipticPosition (n AS INTEGER, T AS DOUBLE, v() AS DOUBLE)

CONST DUMMY.SATELLITE = 5

'Binary stars

TYPE TBINARYORBITEL
  P AS DOUBLE
  T AS DOUBLE
  e AS DOUBLE
  a AS DOUBLE
  Incl AS DOUBLE
  LonPeri AS DOUBLE
  LonNode AS DOUBLE
END TYPE

DECLARE FUNCTION ApparentEccentricity! (El AS TBINARYORBITEL)
DECLARE SUB PositionOfBinary (Year AS DOUBLE, El AS TBINARYORBITEL, Separation AS DOUBLE, PosAngle AS DOUBLE)


'
' I/O functions
'

' Input functions
DECLARE FUNCTION ReadInt% (s AS STRING)
DECLARE FUNCTION ReadReal# (s AS STRING)
DECLARE FUNCTION ReadHMS# (s AS STRING)
DECLARE FUNCTION ReadDMS# (s AS STRING)
DECLARE SUB ReadDate (s AS STRING, Date AS TDATE)

' Output procedures
DECLARE SUB WriteHMS (Time#, n%)
DECLARE SUB WriteDMS (Angle#, n%)
DECLARE SUB WriteDate (Date AS TDATE)

'
' Planetary phenomena
'

' EquinoxSolstice
CONST SPRINGEQUINOX = 0
CONST SUMMERSOLSTICE = 1
CONST AUTUMNEQUINOX = 2
CONST WINTERSOLSTICE = 3

' ConjunctionOpposition
CONST CONJUNCTION = 1
CONST OPPOSITION = 0
CONST INFCONJ = 0
CONST SUPCONJ = 1
CONST EASTELONGATION = 0
CONST WESTELONGATION = 1

' MoonPhase
CONST NEWMOON = 0
CONST FIRSTQUARTER = 1
CONST FULLMOON = 2
CONST LASTQUARTER = 3

' Eclipses
CONST TOTAL = 1
CONST ANNULAR = 2
CONST ANNULARTOTAL = 3
CONST TOTAL.NOT.CENTRAL = 4
CONST ANNULAR.NOT.CENTRAL = 5
CONST PARTIAL = 6

CONST OLDSYSTEM = 1
CONST PENUMBRAL = 7
CONST PARTPENUMBRAL = 8

DECLARE FUNCTION EquinoxSolstice# (Year%, EventID%)
DECLARE FUNCTION FirstkOfYear% (Year%, Planet%, EventID%)
DECLARE FUNCTION ConjunctionOpposition# (Planet%, EventID%, k%)
DECLARE FUNCTION GreatestElongation# (Planet%, EventID%, k%, Elongation#)
DECLARE FUNCTION MoonPhase# (k%, EventID%)
DECLARE FUNCTION NextSolarEclipse# (JD#, Max#, EclipseType%)
DECLARE FUNCTION NextLunarEclipse# (JD#, ConstSystem%, Max#, PartialSemiDuration#, TotalSemiDuration#, EclipseType%)


'
' Physical ephemerides
'

TYPE TMOONPHYSDATA
  l AS SINGLE
  b AS SINGLE   ' Total libration
  Ld AS SINGLE
  Bd AS SINGLE  ' Optical libration
  Ldd AS SINGLE
  Bdd AS SINGLE ' Physical libration
  P AS SINGLE   ' Position angle
  i AS DOUBLE
  k AS SINGLE   ' Phase angle and illuminated fraction
  x AS SINGLE   ' Position angle of the bright limb
  END TYPE

TYPE TSUNPHYSDATA
  P AS SINGLE   ' Position angle
  B0 AS DOUBLE
  L0 AS DOUBLE  ' Heliographic longitude and latitude
  END TYPE

TYPE TMARSPHYSDATA
  DS AS DOUBLE
  DE AS DOUBLE  ' Planetocentric declination of Sun and Earth
  Om AS SINGLE  ' Longitude of the central meridian
  P AS SINGLE   ' Position angle of the axis
  q AS SINGLE
  qq AS SINGLE  ' Position angle and -defect of illumination
  d AS SINGLE   ' Angular diameter in seconds of arc
  END TYPE

TYPE TJUPITERPHYSDATA
  DS AS SINGLE
  DE AS SINGLE  ' Planetocentric declination of Sun and Earth
  Om1 AS SINGLE
  Om2 AS SINGLE ' Longitude of the central meridians (I and II)
  c AS SINGLE   ' Correction for phase to the above
  P AS SINGLE   ' Position angle of the axis
  END TYPE

TYPE TSATURNRINGDATA
  b AS DOUBLE
  Bd AS DOUBLE  ' Planetocentric declination of Earth and Sun
  P AS SINGLE   ' Position angle of the axis
  DeltaU AS SINGLE      ' Needed for magnitude calculation
  aAxis AS SINGLE
  bAxis AS SINGLE       ' Length of axes of rings in arcseconds
  END TYPE

DECLARE FUNCTION MoonSemiDiameter# (DistEarth#)
DECLARE FUNCTION JDOfCarringtonRotation# (c%)
DECLARE FUNCTION CarringtonRotation% (JD#)

DECLARE SUB MoonPhysEphemeris (T#, SGeo AS TSVECTOR, SSun AS TSVECTOR, Obl#, NutLon#, NutObl#, MoonPhysData AS TMOONPHYSDATA)
DECLARE SUB SunPhysEphemeris (T#, l#, Obl#, NutLon#, SunPhysData AS TSUNPHYSDATA)
DECLARE SUB MarsPhysEphemeris (T#, SHelio AS TSVECTOR, SEarth AS TSVECTOR, SGeo AS TSVECTOR, Obl#, NutLon#, NutObl#, MarsPhysData AS TMARSPHYSDATA)
DECLARE SUB JupiterPhysEphemeris (T#, SHelio AS TSVECTOR, SEarth AS TSVECTOR, SGeo AS TSVECTOR, Obl#, NutLon#, NutObl#, JupiterPhysData AS TJUPITERPHYSDATA)
DECLARE SUB SaturnRing (T#, SHelio AS TSVECTOR, SGeo AS TSVECTOR, Obl#, NutLon#, NutObl#, SaturnRingData AS TSATURNRINGDATA)

' Phase angles, magnitudes and semidiameters

DECLARE FUNCTION CalcPhase# (rPS#, rES#, rPE#)
DECLARE FUNCTION CalcPhaseAngle# (rPS#, rES#, rPE#)
DECLARE FUNCTION PlanetSemiDiameter (Planet%, DistEarth#, PolarDiam)

DECLARE SUB CorrectSaturnSemiDiameter (b#, PolarSemiDiameter)

DECLARE FUNCTION PlanetMagnitude (Planet%, rPS#, rPE#, i#, dU#, b#)
DECLARE FUNCTION CometMagnitude (rCS#, rCE#, G, k)
DECLARE FUNCTION AsteroidMagnitude (rAS#, rAE#, PhaseAngle#, H, G)

DECLARE FUNCTION ToBrightness (m)
DECLARE FUNCTION ToMagnitude (b)
DECLARE FUNCTION AddMagnitudes (m1, m2)
DECLARE FUNCTION BrightnessRatio (m1, m2)


'
' Coordinate transformations etc.
'

CONST h0Sun = (-.8333 * DToR)
CONST h0Planet = (-.5667 * DToR)

' RiseSet flags
CONST ALWAYS.ABOVE = 1         ' body is always above horizon
CONST ALWAYS.BELOW = 2         ' body is always below horizon

DECLARE SUB Aberration (T#, Obl#, FKSystem%, RA#, Decl#)
DECLARE FUNCTION AtmRefraction (Altitude#)
DECLARE SUB EclToEqu (l#, b#, Obl#, RA#, Decl#)
DECLARE SUB EquToHor (RA#, Decl#, LST#, ObsLat#, Az#, Alt#)
DECLARE SUB EquToGal (RA#, Decl#, l#, b#)
DECLARE FUNCTION EquinoxCorrection (T#)
DECLARE SUB eTermsOfAberration (FKSystem AS INTEGER, RA AS DOUBLE, Decl AS DOUBLE)
DECLARE SUB FK5ToFK4 (RA AS DOUBLE, Decl AS DOUBLE, dRA AS DOUBLE, dDecl AS DOUBLE)
DECLARE SUB FK4ToFK5 (RA AS DOUBLE, Decl AS DOUBLE, dRA AS DOUBLE, dDecl AS DOUBLE)
DECLARE SUB NutationConst (T#, NutLon#, NutObl#)
DECLARE SUB Nutation (NutLon#, NutObl#, Obl#, RA#, Decl#)
DECLARE FUNCTION Obliquity# (T#)
DECLARE SUB PrecessFK4 (T0#, T1#, RA#, Decl#)
DECLARE SUB PrecessFK5 (T0#, T1#, RA#, Decl#)
DECLARE FUNCTION ParallacticAngle# (RA#, Decl#, ObsLat#, LAST#)
DECLARE SUB DoParallax (Parallax#, LST#, RhoCosPhi#, RhoSinPhi#, RA#, Decl#)
DECLARE SUB DoParallaxHi (sinParallax#, LST#, RhoCosPhi#, RhoSinPhi#, RA#, Decl#)
DECLARE SUB DoParallaxHiAlt (sinParallax#, LST#, RhoCosPhi#, RhoSinPhi#, RA#, Decl#, SD#)
DECLARE FUNCTION MoonSetHeight# (Parallax#)
DECLARE SUB RiseSet (T#, DeltaT#, RA1#, Decl1#, RA2#, Decl2#, RA3#, Decl3#, Height0#, Lon#, Lat#, RISE#, Transit#, Setting#, Flags%)


